unit mTKugeln;

interface

uses      mSuM, mTKugel, mHilfe, Math, Dialogs, SysUtils;
// Uses-Klausel ggf. anpassen

type
   TKugeln = CLASS

     // Attribute
     PRIVATE
        kenntKugel1, kenntKugel2 : TKugel;
     // Methoden
     PRIVATE
        function GetAbstand : Zahl;
     PUBLIC
        constructor init(Kugel1, Kugel2 : TKugel);
        procedure zeichne;
        procedure schieferStoss;
        procedure bewege;
        destructor gibFrei;
        property Abstand : Zahl read GetAbstand;
   end;

implementation

const epsilon = 1.0E-18;

   constructor TKugeln.init(Kugel1, Kugel2 : TKugel);
   begin
        kenntKugel1 := Kugel1;
        kenntKugel2 := Kugel2;
        zeichne;
   end;

   procedure TKugeln.zeichne;
   begin
        kenntKugel1.zeichne;
        kenntKugel2.zeichne;
   end;

   function TKugeln.GetAbstand : Zahl;
   var s : Zahl;
   begin
        s := sqr(kenntKugel1.hPosition-kenntKugel2.hPosition);
        s := s + sqr(kenntKugel1.vPosition-kenntKugel2.vPosition);
        s := sqrt(s);
        Result := s;
   end;

   procedure TKugeln.schieferStoss;
   var v1Vekt, v2Vekt, nVekt, m1Vekt, m2Vekt : TPunkt;
       v1Norm, v2Norm, nNorm, SProd1, SProd2 : Zahl;

       function Betrag(Vekt : TPunkt) : Zahl;
       begin
            if Vekt.x = 0
               then Result := abs(Vekt.y)
               else  if Vekt.x = 0 then Result := abs(Vekt.y)
                                   else Result := sqrt(sqr(Vekt.x)+sqr(Vekt.y));
       end;

       function SkalarProdukt(Vekt1, Vekt2 : TPunkt) : Zahl;
       begin
            Result := Vekt1.x*Vekt2.x + Vekt1.y*Vekt2.y;
       end;

       procedure RichtungsVektor(Winkel : Zahl; var Vekt : TPunkt);
       begin
            Vekt.x := cos(winkel/180*Pi);
            if abs(Vekt.x) <= epsilon then Vekt.x := 0;
            Vekt.y := -1*sin(Winkel/180*Pi);
            if abs(Vekt.y) <= epsilon then Vekt.y := 0;
       end;

       function RichtungsWinkel(Vekt : TPunkt) : Zahl;
       var alpha : Zahl;
       begin
            if abs(Vekt.x) <= epsilon
               then alpha := 90
               else alpha := arccos(Vekt.x)/Pi * 180;
            if ((Vekt.y) < 0) then alpha := -1.*alpha;
            Result := -1*alpha;
       end;

   begin  {schieferStoss}
        nVekt.x := kenntKugel2.hPosition  - kenntKugel1.hPosition;
        nVekt.y := kenntKugel2.vPosition  - kenntKugel1.vPosition;
        nNorm := Betrag(nVekt);
        nVekt.x := nVekt.x/nNorm;
        nVekt.y := nVekt.y/nNorm;
        v1Norm := kenntKugel1.Geschwindigkeit;
        RichtungsVektor(kenntKugel1.Richtung , v1Vekt);
        v1Vekt.x := v1Norm*v1Vekt.x;
        v1Vekt.y := v1Norm*v1Vekt.y;
        v2Norm := kenntKugel2.Geschwindigkeit;
        RichtungsVektor(kenntKugel2.Richtung, v2Vekt);
        v2Vekt.x := v2Norm*v2Vekt.x;
        v2Vekt.y := v2Norm*v2Vekt.y;
        SProd1 := SkalarProdukt(v1Vekt,nVekt);
        m1Vekt.x := v1Vekt.x - SProd1*nVekt.x;
        m1Vekt.y := v1Vekt.y - SProd1*nVekt.y;
        SProd2 := SkalarProdukt(v2Vekt,nVekt);
        m2Vekt.x := v2Vekt.x - SProd2*nVekt.x;
        m2Vekt.y := v2Vekt.y - SProd2*nVekt.y;
        v1Vekt.x := m1Vekt.x + SProd2*nVekt.x;
        v1Vekt.y := m1Vekt.y + SProd2*nVekt.y;
        v1Norm := Betrag(v1Vekt);
        if v1Norm > 0
           then begin
                     v1Vekt.x := v1Vekt.x/v1Norm;
                     v1Vekt.y := v1Vekt.y/v1Norm;
                     kenntKugel1.Richtung := RichtungsWinkel(v1Vekt);
                end
           else kenntKugel1.Richtung := 0;
        kenntKugel1.Geschwindigkeit := v1Norm;
        v2Vekt.x := m2Vekt.x + SProd1*nVekt.x;
        v2Vekt.y := m2Vekt.y + SProd1*nVekt.y;
        v2Norm := Betrag(v2Vekt);
        if v2Norm > 0
           then begin
                     v2Vekt.x := v2Vekt.x/v2Norm;
                     v2Vekt.y := v2Vekt.y/v2Norm;
                     kenntKugel2.Richtung := RichtungsWinkel(v2Vekt);
                end
           else kenntKugel2.Richtung := 0;
        kenntKugel2.Geschwindigkeit := v2Norm;

        //MessageDlg('v1: '+FloatToStr(v1Norm)+#13#10
        //          +'v2: '+FloatToStr(v2Norm)+#13#10
        //          +'Winkel1: '+FloatToStr(kenntKugel1.Richtung)+#13#10
        //          +'Winkel2: '+FloatToStr(kenntKugel2.Richtung), mtInformation,[mbOk], 0);

   end;

   procedure TKugeln.bewege;
   var Kollision : Boolean;
       x1, y1, x2, y2 : Zahl;
       K1Bd, K2Bd, K1K2Bd : Boolean;
       K1str, K2str, K12str : string;
   begin
        kollision := (Abstand <= kenntKugel1.Radius + kenntKugel2.Radius);
        if not Kollision
           then begin
                     kenntKugel1.bewege;
                     kenntKugel2.bewege;
                end
          else
             begin

 {         MessageDlg('Winkel1: '+ FloatToStr(kenntKugel1.Richtung)+#13#10
                  +'v1: '+ FloatToStr(kenntKugel1.Geschwindigkeit)+#13#10
                  +'Winkel2: '+ FloatToStr(kenntKugel2.Richtung)+#13#10
                  +'v2: '+ FloatToStr(kenntKugel2.Geschwindigkeit), mtInformation,[mbOk], 0);
  }
                schieferStoss;


  {     MessageDlg('Winkel1: '+ FloatToStr(kenntKugel1.Richtung)+#13#10
                  +'v1: '+ FloatToStr(kenntKugel1.Geschwindigkeit)+#13#10
                  +'Winkel2: '+ FloatToStr(kenntKugel2.Richtung)+#13#10
                  +'v2: '+ FloatToStr(kenntKugel2.Geschwindigkeit), mtInformation,[mbOk], 0);
   }
                 with kenntKugel2 do
                    with kenntSpielfeld.Rand do
                       begin
                            if (hPosition >= Rechts - Radius)
                               or (hPosition <= Links + Radius)
                              then setzeRichtung (-hatStift.winkel + 180);
                            if (vPosition <= Oben + Radius)
                               or (vPosition >= Unten - Radius)
                              then setzeRichtung (-hatStift.winkel);
                            x2 := hPosition; y2 := vPosition;
                            hatStift.bewegeUm(0.1*Geschwindigkeit);
                       end;

                with kenntKugel1 do
                   with kenntSpielfeld.Rand do
                      begin
                           if (hPosition >= Rechts - Radius)
                              or (hPosition <= Links + Radius)
                             then setzeRichtung (-hatStift.winkel + 180);
                           if (vPosition <= Oben + Radius)
                              or (vPosition >= Unten - Radius)
                             then setzeRichtung (-hatStift.winkel);
                           x1 := hPosition; y1 := vPosition;
                           hatStift.bewegeUm(0.1*Geschwindigkeit);
                      end;

 {        MessageDlg('K1vPos: '+ FloatToStr(kenntKugel1.vPosition)+#13#10
                  +'Winkel1: '+ FloatToStr(kenntKugel1.Richtung)+#13#10
                  +'v1: '+ FloatToStr(kenntKugel1.Geschwindigkeit)+#13#10
                  +'Winkel2: '+ FloatToStr(kenntKugel2.Richtung)+#13#10
                  +'v2: '+ FloatToStr(kenntKugel2.Geschwindigkeit), mtInformation,[mbOk], 0);
  }
                kollision := (Abstand <= kenntKugel1.Radius + kenntKugel2.Radius);
                with kenntKugel1 do begin hPosition := x1; vPosition := y1; end;
                with kenntKugel2 do begin hPosition := x2; vPosition := y2; end;

                if Kollision then schieferStoss;
                with kenntKugel1 do
                                begin
                                     Loesche;
                                     hatStift.bewegeUm(0.1*Geschwindigkeit);
                                     zeichne;
                                end;
                with kenntKugel2 do
                                begin
                                     Loesche;
                                     hatStift.bewegeUm(0.1*Geschwindigkeit);
                                     zeichne;
                                     kenntSpielfeld.zeichne;
                                end;
             end;
   end;

   destructor TKugeln.gibFrei;
   begin
        kenntKugel1.GibFrei;
        kenntKugel2.GibFrei;
   end;
end.
 